@@ -1,17 +1,17 @@
 from collections import defaultdict
 from datetime import date, datetime, timedelta
-from typing import Callable, List
+from typing import Callable, List, Dict, Optional, Type
 from functools import lru_cache, partial
 import traceback
 
 import numpy as np
-from pandas import DataFrame
+from pandas import DataFrame, Series
 import plotly.graph_objects as go
 from plotly.subplots import make_subplots
 
 from vnpy.trader.constant import (Direction, Offset, Exchange,
                                   Interval, Status)
-from vnpy.trader.database import get_database
+from vnpy.trader.database import get_database, BaseDatabase
 from vnpy.trader.object import OrderData, TradeData, BarData, TickData
 from vnpy.trader.utility import round_to
 from vnpy.trader.optimize import (
@@ -35,54 +35,54 @@ from .template import CtaTemplate
 class BacktestingEngine:
     """"""
 
-    engine_type = EngineType.BACKTESTING
-    gateway_name = "BACKTESTING"
+    engine_type: EngineType = EngineType.BACKTESTING
+    gateway_name: str = "BACKTESTING"
 
-    def __init__(self):
+    def __init__(self) -> None:
         """"""
-        self.vt_symbol = ""
-        self.symbol = ""
-        self.exchange = None
-        self.start = None
-        self.end = None
-        self.rate = 0
-        self.slippage = 0
-        self.size = 1
-        self.pricetick = 0
-        self.capital = 1_000_000
+        self.vt_symbol: str = ""
+        self.symbol: str = ""
+        self.exchange: Exchange = None
+        self.start: datetime = None
+        self.end: datetime = None
+        self.rate: float = 0
+        self.slippage: float = 0
+        self.size: float = 1
+        self.pricetick: float = 0
+        self.capital: int = 1_000_000
         self.risk_free: float = 0
         self.annual_days: int = 240
-        self.mode = BacktestingMode.BAR
-        self.inverse = False
+        self.mode: BacktestingMode = BacktestingMode.BAR
+        self.inverse: bool = False
 
-        self.strategy_class = None
-        self.strategy = None
+        self.strategy_class: Type[CtaTemplate] = None
+        self.strategy: CtaTemplate = None
         self.tick: TickData
         self.bar: BarData
-        self.datetime = None
+        self.datetime: datetime = None
 
-        self.interval = None
-        self.days = 0
-        self.callback = None
-        self.history_data = []
+        self.interval: Interval = None
+        self.days: int = 0
+        self.callback: Callable = None
+        self.history_data: list = []
 
-        self.stop_order_count = 0
-        self.stop_orders = {}
-        self.active_stop_orders = {}
+        self.stop_order_count: int = 0
+        self.stop_orders: Dict[str, StopOrder] = {}
+        self.active_stop_orders: Dict[str, StopOrder] = {}
 
-        self.limit_order_count = 0
-        self.limit_orders = {}
-        self.active_limit_orders = {}
+        self.limit_order_count: int = 0
+        self.limit_orders: Dict[str, OrderData] = {}
+        self.active_limit_orders: Dict[str, OrderData] = {}
 
-        self.trade_count = 0
-        self.trades = {}
+        self.trade_count: int = 0
+        self.trades: Dict[str, TradeData] = {}
 
-        self.logs = []
+        self.logs: list = []
 
-        self.daily_results = {}
-        self.daily_df = None
+        self.daily_results: Dict[date, DailyResult] = {}
+        self.daily_df: DataFrame = None
 
-    def clear_data(self):
+    def clear_data(self) -> None:
         """
         Clear all data of last backtesting.
         """
@@ -120,7 +120,7 @@ class BacktestingEngine:
         inverse: bool = False,
         risk_free: float = 0,
         annual_days: int = 240
-    ):
+    ) -> None:
         """"""
         self.mode = mode
         self.vt_symbol = vt_symbol
@@ -141,14 +141,14 @@ class BacktestingEngine:
         self.risk_free = risk_free
         self.annual_days = annual_days
 
-    def add_strategy(self, strategy_class: type, setting: dict):
+    def add_strategy(self, strategy_class: Type[CtaTemplate], setting: dict) -> None:
         """"""
         self.strategy_class = strategy_class
         self.strategy = strategy_class(
             self, strategy_class.__name__, self.vt_symbol, setting
         )
 
-    def load_data(self):
+    def load_data(self) -> None:
         """"""
         self.output("开始加载历史数据")
 
@@ -162,23 +162,23 @@ class BacktestingEngine:
         self.history_data.clear()       # Clear previously loaded history data
 
         # Load 30 days of data each time and allow for progress update
-        total_days = (self.end - self.start).days
-        progress_days = max(int(total_days / 10), 1)
-        progress_delta = timedelta(days=progress_days)
-        interval_delta = INTERVAL_DELTA_MAP[self.interval]
+        total_days: int = (self.end - self.start).days
+        progress_days: int = max(int(total_days / 10), 1)
+        progress_delta: timedelta = timedelta(days=progress_days)
+        interval_delta: timedelta = INTERVAL_DELTA_MAP[self.interval]
 
-        start = self.start
-        end = self.start + progress_delta
+        start: datetime = self.start
+        end: datetime = self.start + progress_delta
         progress = 0
 
         while start < self.end:
-            progress_bar = "#" * int(progress * 10 + 1)
+            progress_bar: str = "#" * int(progress * 10 + 1)
             self.output(f"加载进度：{progress_bar} [{progress:.0%}]")
 
-            end = min(end, self.end)  # Make sure end time stays within set range
+            end: datetime = min(end, self.end)  # Make sure end time stays within set range
 
             if self.mode == BacktestingMode.BAR:
-                data = load_bar_data(
+                data: List[BarData] = load_bar_data(
                     self.symbol,
                     self.exchange,
                     self.interval,
@@ -186,7 +186,7 @@ class BacktestingEngine:
                     end
                 )
             else:
-                data = load_tick_data(
+                data: List[TickData] = load_tick_data(
                     self.symbol,
                     self.exchange,
                     start,
@@ -203,7 +203,7 @@ class BacktestingEngine:
 
         self.output(f"历史数据加载完成，数据量：{len(self.history_data)}")
 
-    def run_backtesting(self):
+    def run_backtesting(self) -> None:
         """"""
         if self.mode == BacktestingMode.BAR:
             func = self.new_bar
@@ -213,8 +213,8 @@ class BacktestingEngine:
         self.strategy.on_init()
 
         # Use the first [days] of history data for initializing strategy
-        day_count = 0
-        ix = 0
+        day_count: int = 0
+        ix: int = 0
 
         for ix, data in enumerate(self.history_data):
             if self.datetime and data.datetime.day != self.datetime.day:
@@ -239,16 +239,16 @@ class BacktestingEngine:
         self.output("开始回放历史数据")
 
         # Use the rest of history data for running backtesting
-        backtesting_data = self.history_data[ix:]
+        backtesting_data: list = self.history_data[ix:]
         if len(backtesting_data) <= 1:
             self.output("历史数据不足，回测终止")
             return
 
-        total_size = len(backtesting_data)
-        batch_size = max(int(total_size / 10), 1)
+        total_size: int = len(backtesting_data)
+        batch_size: int = max(int(total_size / 10), 1)
 
         for ix, i in enumerate(range(0, total_size, batch_size)):
-            batch_data = backtesting_data[i: i + batch_size]
+            batch_data: list = backtesting_data[i: i + batch_size]
             for data in batch_data:
                 try:
                     func(data)
@@ -258,13 +258,13 @@ class BacktestingEngine:
                     return
 
             progress = min(ix / 10, 1)
-            progress_bar = "=" * (ix + 1)
+            progress_bar: str = "=" * (ix + 1)
             self.output(f"回放进度：{progress_bar} [{progress:.0%}]")
 
         self.strategy.on_stop()
         self.output("历史数据回放结束")
 
-    def calculate_result(self):
+    def calculate_result(self) -> DataFrame:
         """"""
         self.output("开始计算逐日盯市盈亏")
 
@@ -274,8 +274,8 @@ class BacktestingEngine:
 
         # Add trade data into daily reuslt.
         for trade in self.trades.values():
-            d = trade.datetime.date()
-            daily_result = self.daily_results[d]
+            d: date = trade.datetime.date()
+            daily_result: DailyResult = self.daily_results[d]
             daily_result.add_trade(trade)
 
         # Calculate daily result by iteration.
@@ -296,7 +296,7 @@ class BacktestingEngine:
             start_pos = daily_result.end_pos
 
         # Generate dataframe
-        results = defaultdict(list)
+        results: defaultdict = defaultdict(list)
 
         for daily_result in self.daily_results.values():
             for key, value in daily_result.__dict__.items():
@@ -307,48 +307,48 @@ class BacktestingEngine:
         self.output("逐日盯市盈亏计算完成")
         return self.daily_df
 
-    def calculate_statistics(self, df: DataFrame = None, output=True):
+    def calculate_statistics(self, df: DataFrame = None, output=True) -> dict:
         """"""
         self.output("开始计算策略统计指标")
 
         # Check DataFrame input exterior
         if df is None:
-            df = self.daily_df
+            df: DataFrame = self.daily_df
 
         # Check for init DataFrame
         if df is None:
             # Set all statistics to 0 if no trade.
-            start_date = ""
-            end_date = ""
-            total_days = 0
-            profit_days = 0
-            loss_days = 0
-            end_balance = 0
-            max_drawdown = 0
-            max_ddpercent = 0
-            max_drawdown_duration = 0
-            total_net_pnl = 0
-            daily_net_pnl = 0
-            total_commission = 0
-            daily_commission = 0
-            total_slippage = 0
-            daily_slippage = 0
-            total_turnover = 0
-            daily_turnover = 0
-            total_trade_count = 0
-            daily_trade_count = 0
-            total_return = 0
-            annual_return = 0
-            daily_return = 0
-            return_std = 0
-            sharpe_ratio = 0
-            return_drawdown_ratio = 0
+            start_date: str = ""
+            end_date: str = ""
+            total_days: int = 0
+            profit_days: int = 0
+            loss_days: int = 0
+            end_balance: float = 0
+            max_drawdown: float = 0
+            max_ddpercent: float = 0
+            max_drawdown_duration: int = 0
+            total_net_pnl: float = 0
+            daily_net_pnl: float = 0
+            total_commission: float = 0
+            daily_commission: float = 0
+            total_slippage: float = 0
+            daily_slippage: float = 0
+            total_turnover: float = 0
+            daily_turnover: float = 0
+            total_trade_count: int = 0
+            daily_trade_count: int = 0
+            total_return: float = 0
+            annual_return: float = 0
+            daily_return: float = 0
+            return_std: float = 0
+            sharpe_ratio: float = 0
+            return_drawdown_ratio: float = 0
         else:
             # Calculate balance related time series data
             df["balance"] = df["net_pnl"].cumsum() + self.capital
 
             # When balance falls below 0, set daily return to 0
-            pre_balance = df["balance"].shift(1)
+            pre_balance: Series = df["balance"].shift(1)
             pre_balance.iloc[0] = self.capital
             x = df["balance"] / pre_balance
             x[x <= 0] = np.nan
@@ -365,9 +365,9 @@ class BacktestingEngine:
             start_date = df.index[0]
             end_date = df.index[-1]
 
-            total_days = len(df)
-            profit_days = len(df[df["net_pnl"] > 0])
-            loss_days = len(df[df["net_pnl"] < 0])
+            total_days: int = len(df)
+            profit_days: int = len(df[df["net_pnl"] > 0])
+            loss_days: int = len(df[df["net_pnl"] < 0])
 
             end_balance = df["balance"].iloc[-1]
             max_drawdown = df["drawdown"].min()
@@ -376,37 +376,37 @@ class BacktestingEngine:
 
             if isinstance(max_drawdown_end, date):
                 max_drawdown_start = df["balance"][:max_drawdown_end].idxmax()
-                max_drawdown_duration = (max_drawdown_end - max_drawdown_start).days
+                max_drawdown_duration: int = (max_drawdown_end - max_drawdown_start).days
             else:
-                max_drawdown_duration = 0
+                max_drawdown_duration: int = 0
 
-            total_net_pnl = df["net_pnl"].sum()
-            daily_net_pnl = total_net_pnl / total_days
+            total_net_pnl: float = df["net_pnl"].sum()
+            daily_net_pnl: float = total_net_pnl / total_days
 
-            total_commission = df["commission"].sum()
-            daily_commission = total_commission / total_days
+            total_commission: float = df["commission"].sum()
+            daily_commission: float = total_commission / total_days
 
-            total_slippage = df["slippage"].sum()
-            daily_slippage = total_slippage / total_days
+            total_slippage: float = df["slippage"].sum()
+            daily_slippage: float = total_slippage / total_days
 
-            total_turnover = df["turnover"].sum()
-            daily_turnover = total_turnover / total_days
+            total_turnover: float = df["turnover"].sum()
+            daily_turnover: float = total_turnover / total_days
 
-            total_trade_count = df["trade_count"].sum()
-            daily_trade_count = total_trade_count / total_days
+            total_trade_count: int = df["trade_count"].sum()
+            daily_trade_count: int = total_trade_count / total_days
 
-            total_return = (end_balance / self.capital - 1) * 100
-            annual_return = total_return / total_days * self.annual_days
-            daily_return = df["return"].mean() * 100
-            return_std = df["return"].std() * 100
+            total_return: float = (end_balance / self.capital - 1) * 100
+            annual_return: float = total_return / total_days * self.annual_days
+            daily_return: float = df["return"].mean() * 100
+            return_std: float = df["return"].std() * 100
 
             if return_std:
-                daily_risk_free = self.risk_free / np.sqrt(self.annual_days)
-                sharpe_ratio = (daily_return - daily_risk_free) / return_std * np.sqrt(self.annual_days)
+                daily_risk_free: float = self.risk_free / np.sqrt(self.annual_days)
+                sharpe_ratio: float = (daily_return - daily_risk_free) / return_std * np.sqrt(self.annual_days)
             else:
-                sharpe_ratio = 0
+                sharpe_ratio: float = 0
 
-            return_drawdown_ratio = -total_return / max_ddpercent
+            return_drawdown_ratio: float = -total_return / max_ddpercent
 
         # Output
         if output:
@@ -444,7 +444,7 @@ class BacktestingEngine:
             self.output(f"Sharpe Ratio：\t{sharpe_ratio:,.2f}")
             self.output(f"收益回撤比：\t{return_drawdown_ratio:,.2f}")
 
-        statistics = {
+        statistics: dict = {
             "start_date": start_date,
             "end_date": end_date,
             "total_days": total_days,
@@ -482,11 +482,11 @@ class BacktestingEngine:
         self.output("策略统计指标计算完成")
         return statistics
 
-    def show_chart(self, df: DataFrame = None):
+    def show_chart(self, df: DataFrame = None) -> None:
         """"""
         # Check DataFrame input exterior
         if df is None:
-            df = self.daily_df
+            df: DataFrame = self.daily_df
 
         # Check for init DataFrame
         if df is None:
@@ -505,6 +505,7 @@ class BacktestingEngine:
             mode="lines",
             name="Balance"
         )
+
         drawdown_scatter = go.Scatter(
             x=df.index,
             y=df["drawdown"],
@@ -524,13 +525,13 @@ class BacktestingEngine:
         fig.update_layout(height=1000, width=1000)
         fig.show()
 
-    def run_bf_optimization(self, optimization_setting: OptimizationSetting, output=True):
+    def run_bf_optimization(self, optimization_setting: OptimizationSetting, output=True) -> list:
         """"""
         if not check_optimization_setting(optimization_setting):
             return
 
         evaluate_func: callable = wrap_evaluate(self, optimization_setting.target_name)
-        results = run_bf_optimization(
+        results: list = run_bf_optimization(
             evaluate_func,
             optimization_setting,
             get_target_value,
@@ -546,13 +547,13 @@ class BacktestingEngine:
 
     run_optimization = run_bf_optimization
 
-    def run_ga_optimization(self, optimization_setting: OptimizationSetting, output=True):
+    def run_ga_optimization(self, optimization_setting: OptimizationSetting, output=True) -> list:
         """"""
         if not check_optimization_setting(optimization_setting):
             return
 
         evaluate_func: callable = wrap_evaluate(self, optimization_setting.target_name)
-        results = run_ga_optimization(
+        results: list = run_ga_optimization(
             evaluate_func,
             optimization_setting,
             get_target_value,
@@ -566,17 +567,17 @@ class BacktestingEngine:
 
         return results
 
-    def update_daily_close(self, price: float):
+    def update_daily_close(self, price: float) -> None:
         """"""
-        d = self.datetime.date()
+        d: date = self.datetime.date()
 
-        daily_result = self.daily_results.get(d, None)
+        daily_result: Optional[DailyResult] = self.daily_results.get(d, None)
         if daily_result:
             daily_result.close_price = price
         else:
             self.daily_results[d] = DailyResult(d, price)
 
-    def new_bar(self, bar: BarData):
+    def new_bar(self, bar: BarData) -> None:
         """"""
         self.bar = bar
         self.datetime = bar.datetime
@@ -587,7 +588,7 @@ class BacktestingEngine:
 
         self.update_daily_close(bar.close_price)
 
-    def new_tick(self, tick: TickData):
+    def new_tick(self, tick: TickData) -> None:
         """"""
         self.tick = tick
         self.datetime = tick.datetime
@@ -598,7 +599,7 @@ class BacktestingEngine:
 
         self.update_daily_close(tick.last_price)
 
-    def cross_limit_order(self):
+    def cross_limit_order(self) -> None:
         """
         Cross limit order with last bar/tick data.
         """
@@ -620,13 +621,13 @@ class BacktestingEngine:
                 self.strategy.on_order(order)
 
             # Check whether limit orders can be filled.
-            long_cross = (
+            long_cross: bool = (
                 order.direction == Direction.LONG
                 and order.price >= long_cross_price
                 and long_cross_price > 0
             )
 
-            short_cross = (
+            short_cross: bool = (
                 order.direction == Direction.SHORT
                 and order.price <= short_cross_price
                 and short_cross_price > 0
@@ -653,7 +654,7 @@ class BacktestingEngine:
                 trade_price = max(order.price, short_best_price)
                 pos_change = -order.volume
 
-            trade = TradeData(
+            trade: TradeData = TradeData(
                 symbol=order.symbol,
                 exchange=order.exchange,
                 orderid=order.orderid,
@@ -671,7 +672,7 @@ class BacktestingEngine:
 
             self.trades[trade.vt_tradeid] = trade
 
-    def cross_stop_order(self):
+    def cross_stop_order(self) -> None:
         """
         Cross stop order with last bar/tick data.
         """
@@ -688,12 +689,12 @@ class BacktestingEngine:
 
         for stop_order in list(self.active_stop_orders.values()):
             # Check whether stop order can be triggered.
-            long_cross = (
+            long_cross: bool = (
                 stop_order.direction == Direction.LONG
                 and stop_order.price <= long_cross_price
             )
 
-            short_cross = (
+            short_cross: bool = (
                 stop_order.direction == Direction.SHORT
                 and stop_order.price >= short_cross_price
             )
@@ -704,7 +705,7 @@ class BacktestingEngine:
             # Create order data.
             self.limit_order_count += 1
 
-            order = OrderData(
+            order: OrderData = OrderData(
                 symbol=self.symbol,
                 exchange=self.exchange,
                 orderid=str(self.limit_order_count),
@@ -730,7 +731,7 @@ class BacktestingEngine:
 
             self.trade_count += 1
 
-            trade = TradeData(
+            trade: TradeData = TradeData(
                 symbol=order.symbol,
                 exchange=order.exchange,
                 orderid=order.orderid,
@@ -788,13 +789,13 @@ class BacktestingEngine:
         stop: bool,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """"""
-        price = round_to(price, self.pricetick)
+        price: float = round_to(price, self.pricetick)
         if stop:
-            vt_orderid = self.send_stop_order(direction, offset, price, volume)
+            vt_orderid: str = self.send_stop_order(direction, offset, price, volume)
         else:
-            vt_orderid = self.send_limit_order(direction, offset, price, volume)
+            vt_orderid: str = self.send_limit_order(direction, offset, price, volume)
         return [vt_orderid]
 
     def send_stop_order(
@@ -803,11 +804,11 @@ class BacktestingEngine:
         offset: Offset,
         price: float,
         volume: float
-    ):
+    ) -> str:
         """"""
         self.stop_order_count += 1
 
-        stop_order = StopOrder(
+        stop_order: StopOrder = StopOrder(
             vt_symbol=self.vt_symbol,
             direction=direction,
             offset=offset,
@@ -829,11 +830,11 @@ class BacktestingEngine:
         offset: Offset,
         price: float,
         volume: float
-    ):
+    ) -> str:
         """"""
         self.limit_order_count += 1
 
-        order = OrderData(
+        order: OrderData = OrderData(
             symbol=self.symbol,
             exchange=self.exchange,
             orderid=str(self.limit_order_count),
@@ -851,7 +852,7 @@ class BacktestingEngine:
 
         return order.vt_orderid
 
-    def cancel_order(self, strategy: CtaTemplate, vt_orderid: str):
+    def cancel_order(self, strategy: CtaTemplate, vt_orderid: str) -> None:
         """
         Cancel order by vt_orderid.
         """
@@ -860,92 +861,92 @@ class BacktestingEngine:
         else:
             self.cancel_limit_order(strategy, vt_orderid)
 
-    def cancel_stop_order(self, strategy: CtaTemplate, vt_orderid: str):
+    def cancel_stop_order(self, strategy: CtaTemplate, vt_orderid: str) -> None:
         """"""
         if vt_orderid not in self.active_stop_orders:
             return
-        stop_order = self.active_stop_orders.pop(vt_orderid)
+        stop_order: StopOrder = self.active_stop_orders.pop(vt_orderid)
 
         stop_order.status = StopOrderStatus.CANCELLED
         self.strategy.on_stop_order(stop_order)
 
-    def cancel_limit_order(self, strategy: CtaTemplate, vt_orderid: str):
+    def cancel_limit_order(self, strategy: CtaTemplate, vt_orderid: str) -> None:
         """"""
         if vt_orderid not in self.active_limit_orders:
             return
-        order = self.active_limit_orders.pop(vt_orderid)
+        order: OrderData = self.active_limit_orders.pop(vt_orderid)
 
         order.status = Status.CANCELLED
         self.strategy.on_order(order)
 
-    def cancel_all(self, strategy: CtaTemplate):
+    def cancel_all(self, strategy: CtaTemplate) -> None:
         """
         Cancel all orders, both limit and stop.
         """
-        vt_orderids = list(self.active_limit_orders.keys())
+        vt_orderids: list = list(self.active_limit_orders.keys())
         for vt_orderid in vt_orderids:
             self.cancel_limit_order(strategy, vt_orderid)
 
-        stop_orderids = list(self.active_stop_orders.keys())
+        stop_orderids: list = list(self.active_stop_orders.keys())
         for vt_orderid in stop_orderids:
             self.cancel_stop_order(strategy, vt_orderid)
 
-    def write_log(self, msg: str, strategy: CtaTemplate = None):
+    def write_log(self, msg: str, strategy: CtaTemplate = None) -> None:
         """
         Write log message.
         """
-        msg = f"{self.datetime}\t{msg}"
+        msg: str = f"{self.datetime}\t{msg}"
         self.logs.append(msg)
 
-    def send_email(self, msg: str, strategy: CtaTemplate = None):
+    def send_email(self, msg: str, strategy: CtaTemplate = None) -> None:
         """
         Send email to default receiver.
         """
         pass
 
-    def sync_strategy_data(self, strategy: CtaTemplate):
+    def sync_strategy_data(self, strategy: CtaTemplate) -> None:
         """
         Sync strategy data into json file.
         """
         pass
 
-    def get_engine_type(self):
+    def get_engine_type(self) -> EngineType:
         """
         Return engine type.
         """
         return self.engine_type
 
-    def get_pricetick(self, strategy: CtaTemplate):
+    def get_pricetick(self, strategy: CtaTemplate) -> float:
         """
         Return contract pricetick data.
         """
         return self.pricetick
 
-    def put_strategy_event(self, strategy: CtaTemplate):
+    def put_strategy_event(self, strategy: CtaTemplate) -> None:
         """
         Put an event to update strategy status.
         """
         pass
 
-    def output(self, msg):
+    def output(self, msg) -> None:
         """
         Output message of backtesting engine.
         """
         print(f"{datetime.now()}\t{msg}")
 
-    def get_all_trades(self):
+    def get_all_trades(self) -> list:
         """
         Return all trade data of current backtesting result.
         """
         return list(self.trades.values())
 
-    def get_all_orders(self):
+    def get_all_orders(self) -> list:
         """
         Return all limit order data of current backtesting result.
         """
         return list(self.limit_orders.values())
 
-    def get_all_daily_results(self):
+    def get_all_daily_results(self) -> list:
         """
         Return all daily result data.
         """
@@ -955,28 +956,28 @@ class BacktestingEngine:
 class DailyResult:
     """"""
 
-    def __init__(self, date: date, close_price: float):
+    def __init__(self, date: date, close_price: float) -> None:
         """"""
-        self.date = date
-        self.close_price = close_price
-        self.pre_close = 0
+        self.date: date = date
+        self.close_price: float = close_price
+        self.pre_close: float = 0
 
-        self.trades = []
-        self.trade_count = 0
+        self.trades: List[TradeData] = []
+        self.trade_count: int = 0
 
         self.start_pos = 0
         self.end_pos = 0
 
-        self.turnover = 0
-        self.commission = 0
-        self.slippage = 0
+        self.turnover: float = 0
+        self.commission: float = 0
+        self.slippage: float = 0
 
-        self.trading_pnl = 0
-        self.holding_pnl = 0
-        self.total_pnl = 0
-        self.net_pnl = 0
+        self.trading_pnl: float = 0
+        self.holding_pnl: float = 0
+        self.total_pnl: float = 0
+        self.net_pnl: float = 0
 
-    def add_trade(self, trade: TradeData):
+    def add_trade(self, trade: TradeData) -> None:
         """"""
         self.trades.append(trade)
 
@@ -988,7 +989,7 @@ class DailyResult:
         rate: float,
         slippage: float,
         inverse: bool
-    ):
+    ) -> None:
         """"""
         # If no pre_close provided on the first day,
         # use value 1 to avoid zero division error
@@ -1004,7 +1005,7 @@ class DailyResult:
         if not inverse:     # For normal contract
             self.holding_pnl = self.start_pos * \
                 (self.close_price - self.pre_close) * size
-        else:               # For crypto currency inverse contract
+        else:               # For inverse contract
             self.holding_pnl = self.start_pos * \
                 (1 / self.pre_close - 1 / self.close_price) * size
 
@@ -1021,13 +1022,13 @@ class DailyResult:
 
             # For normal contract
             if not inverse:
-                turnover = trade.volume * size * trade.price
+                turnover: float = trade.volume * size * trade.price
                 self.trading_pnl += pos_change * \
                     (self.close_price - trade.price) * size
                 self.slippage += trade.volume * size * slippage
-            # For crypto currency inverse contract
+            # For inverse contract
             else:
-                turnover = trade.volume * size / trade.price
+                turnover: float = trade.volume * size / trade.price
                 self.trading_pnl += pos_change * \
                     (1 / trade.price - 1 / self.close_price) * size
                 self.slippage += trade.volume * size * slippage / (trade.price ** 2)
@@ -1047,9 +1048,9 @@ def load_bar_data(
     interval: Interval,
     start: datetime,
     end: datetime
-):
+) -> List[BarData]:
     """"""
-    database = get_database()
+    database: BaseDatabase = get_database()
 
     return database.load_bar_data(
         symbol, exchange, interval, start, end
@@ -1062,9 +1063,9 @@ def load_tick_data(
     exchange: Exchange,
     start: datetime,
     end: datetime
-):
+) -> List[TickData]:
     """"""
-    database = get_database()
+    database: BaseDatabase = get_database()
 
     return database.load_tick_data(
         symbol, exchange, start, end
@@ -1086,11 +1087,11 @@ def evaluate(
     mode: BacktestingMode,
     inverse: bool,
     setting: dict
-):
+) -> tuple:
     """
     Function for running in multiprocessing.pool
     """
-    engine = BacktestingEngine()
+    engine: BacktestingEngine = BacktestingEngine()
 
     engine.set_parameters(
         vt_symbol=vt_symbol,
@@ -1110,9 +1111,9 @@ def evaluate(
     engine.load_data()
     engine.run_backtesting()
     engine.calculate_result()
-    statistics = engine.calculate_statistics(output=False)
+    statistics: dict = engine.calculate_statistics(output=False)
 
-    target_value = statistics[target_name]
+    target_value: float = statistics[target_name]
     return (str(setting), target_value, statistics)
 
 
