@@ -1,15 +1,15 @@
-""""""
-
 import importlib
 import traceback
 from collections import defaultdict
 from pathlib import Path
-from typing import Any, Callable, List
+from types import ModuleType
+from typing import Any, Callable, Dict, List, Optional, Type
 from datetime import datetime, timedelta
 from concurrent.futures import ThreadPoolExecutor
 from copy import copy
 from tzlocal import get_localzone
 from glob import glob
+from concurrent.futures import Future
 
 from vnpy.event import Event, EventEngine
 from vnpy.trader.engine import BaseEngine, MainEngine
@@ -17,10 +17,14 @@ from vnpy.trader.object import (
     OrderRequest,
     SubscribeRequest,
     HistoryRequest,
+    CancelRequest,
     LogData,
     TickData,
     BarData,
-    ContractData
+    OrderData,
+    TradeData,
+    ContractData,
+    PositionData
 )
 from vnpy.trader.event import (
     EVENT_TICK,
@@ -54,7 +58,8 @@ from .base import (
 from .template import CtaTemplate
 
 
-STOP_STATUS_MAP = {
+# 停止单状态映射
+STOP_STATUS_MAP: Dict[Status, StopOrderStatus] = {
     Status.SUBMITTING: StopOrderStatus.WAITING,
     Status.NOTTRADED: StopOrderStatus.WAITING,
     Status.PARTTRADED: StopOrderStatus.TRIGGERED,
@@ -69,46 +74,44 @@ LOCAL_TZ = get_localzone()
 class CtaEngine(BaseEngine):
     """"""
 
-    engine_type = EngineType.LIVE  # live trading engine
+    engine_type: EngineType = EngineType.LIVE  # live trading engine
 
-    setting_filename = "cta_strategy_setting.json"
-    data_filename = "cta_strategy_data.json"
+    setting_filename: str = "cta_strategy_setting.json"
+    data_filename: str = "cta_strategy_data.json"
 
-    def __init__(self, main_engine: MainEngine, event_engine: EventEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         """"""
         super(CtaEngine, self).__init__(
             main_engine, event_engine, APP_NAME)
 
-        self.strategy_setting = {}  # strategy_name: dict
-        self.strategy_data = {}     # strategy_name: dict
+        self.strategy_setting: dict = {}  # strategy_name: dict
+        self.strategy_data: dict = {}     # strategy_name: dict
 
-        self.classes = {}           # class_name: stategy_class
-        self.strategies = {}        # strategy_name: strategy
+        self.classes: dict = {}           # class_name: stategy_class
+        self.strategies: dict = {}        # strategy_name: strategy
 
-        self.symbol_strategy_map = defaultdict(
+        self.symbol_strategy_map: defaultdict = defaultdict(
             list)                   # vt_symbol: strategy list
-        self.orderid_strategy_map = {}  # vt_orderid: strategy
-        self.strategy_orderid_map = defaultdict(
+        self.orderid_strategy_map: dict = {}  # vt_orderid: strategy
+        self.strategy_orderid_map: defaultdict = defaultdict(
             set)                    # strategy_name: orderid list
 
-        self.stop_order_count = 0   # for generating stop_orderid
-        self.stop_orders = {}       # stop_orderid: stop_order
+        self.stop_order_count: int = 0   # for generating stop_orderid
+        self.stop_orders: Dict[str, StopOrder] = {}       # stop_orderid: stop_order
 
-        self.init_executor = ThreadPoolExecutor(max_workers=1)
+        self.init_executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=1)
 
-        self.rq_client = None
-        self.rq_symbols = set()
+        self.rq_symbols: set = set()
 
-        self.vt_tradeids = set()    # for filtering duplicate trade
+        self.vt_tradeids: set = set()    # for filtering duplicate trade
 
-        self.offset_converter = OffsetConverter(self.main_engine)
+        self.offset_converter: OffsetConverter = OffsetConverter(self.main_engine)
 
         self.database: BaseDatabase = get_database()
         self.datafeed: BaseDatafeed = get_datafeed()
 
-    def init_engine(self):
-        """
-        """
+    def init_engine(self) -> None:
+        """"""
         self.init_datafeed()
         self.load_strategy_class()
         self.load_strategy_setting()
@@ -116,46 +119,46 @@ class CtaEngine(BaseEngine):
         self.register_event()
         self.write_log("CTA策略引擎初始化成功")
 
-    def close(self):
+    def close(self) -> None:
         """"""
         self.stop_all_strategies()
 
-    def register_event(self):
+    def register_event(self) -> None:
         """"""
         self.event_engine.register(EVENT_TICK, self.process_tick_event)
         self.event_engine.register(EVENT_ORDER, self.process_order_event)
         self.event_engine.register(EVENT_TRADE, self.process_trade_event)
         self.event_engine.register(EVENT_POSITION, self.process_position_event)
 
-    def init_datafeed(self):
+    def init_datafeed(self) -> None:
         """
         Init datafeed client.
         """
-        result = self.datafeed.init()
+        result: bool = self.datafeed.init()
         if result:
             self.write_log("数据服务初始化成功")
 
     def query_bar_from_datafeed(
         self, symbol: str, exchange: Exchange, interval: Interval, start: datetime, end: datetime
-    ):
+    ) -> List[BarData]:
         """
         Query bar data from datafeed.
         """
-        req = HistoryRequest(
+        req: HistoryRequest = HistoryRequest(
             symbol=symbol,
             exchange=exchange,
             interval=interval,
             start=start,
             end=end
         )
-        data = self.datafeed.query_bar_history(req)
+        data: List[BarData] = self.datafeed.query_bar_history(req)
         return data
 
-    def process_tick_event(self, event: Event):
+    def process_tick_event(self, event: Event) -> None:
         """"""
-        tick = event.data
+        tick: TickData = event.data
 
-        strategies = self.symbol_strategy_map[tick.vt_symbol]
+        strategies: list = self.symbol_strategy_map[tick.vt_symbol]
         if not strategies:
             return
 
@@ -165,24 +168,24 @@ class CtaEngine(BaseEngine):
             if strategy.inited:
                 self.call_strategy_func(strategy, strategy.on_tick, tick)
 
-    def process_order_event(self, event: Event):
+    def process_order_event(self, event: Event) -> None:
         """"""
-        order = event.data
+        order: OrderData = event.data
 
         self.offset_converter.update_order(order)
 
-        strategy = self.orderid_strategy_map.get(order.vt_orderid, None)
+        strategy: Optional[type] = self.orderid_strategy_map.get(order.vt_orderid, None)
         if not strategy:
             return
 
         # Remove vt_orderid if order is no longer active.
-        vt_orderids = self.strategy_orderid_map[strategy.strategy_name]
+        vt_orderids: list = self.strategy_orderid_map[strategy.strategy_name]
         if order.vt_orderid in vt_orderids and not order.is_active():
             vt_orderids.remove(order.vt_orderid)
 
         # For server stop order, call strategy on_stop_order function
         if order.type == OrderType.STOP:
-            so = StopOrder(
+            so: StopOrder = StopOrder(
                 vt_symbol=order.vt_symbol,
                 direction=order.direction,
                 offset=order.offset,
@@ -199,9 +202,9 @@ class CtaEngine(BaseEngine):
         # Call strategy on_order function
         self.call_strategy_func(strategy, strategy.on_order, order)
 
-    def process_trade_event(self, event: Event):
+    def process_trade_event(self, event: Event) -> None:
         """"""
-        trade = event.data
+        trade: TradeData = event.data
 
         # Filter duplicate trade push
         if trade.vt_tradeid in self.vt_tradeids:
@@ -210,7 +213,7 @@ class CtaEngine(BaseEngine):
 
         self.offset_converter.update_trade(trade)
 
-        strategy = self.orderid_strategy_map.get(trade.vt_orderid, None)
+        strategy: Optional[type] = self.orderid_strategy_map.get(trade.vt_orderid, None)
         if not strategy:
             return
 
@@ -228,13 +231,13 @@ class CtaEngine(BaseEngine):
         # Update GUI
         self.put_strategy_event(strategy)
 
-    def process_position_event(self, event: Event):
+    def process_position_event(self, event: Event) -> None:
         """"""
-        position = event.data
+        position: PositionData = event.data
 
         self.offset_converter.update_position(position)
 
-    def check_stop_order(self, tick: TickData):
+    def check_stop_order(self, tick: TickData) -> None:
         """"""
         for stop_order in list(self.stop_orders.values()):
             if stop_order.vt_symbol != tick.vt_symbol:
@@ -248,7 +251,7 @@ class CtaEngine(BaseEngine):
             )
 
             if long_triggered or short_triggered:
-                strategy = self.strategies[stop_order.strategy_name]
+                strategy: CtaTemplate = self.strategies[stop_order.strategy_name]
 
                 # To get excuted immediately after stop order is
                 # triggered, use limit price if available, otherwise
@@ -264,9 +267,9 @@ class CtaEngine(BaseEngine):
                     else:
                         price = tick.bid_price_5
 
-                contract = self.main_engine.get_contract(stop_order.vt_symbol)
+                contract: Optional[ContractData] = self.main_engine.get_contract(stop_order.vt_symbol)
 
-                vt_orderids = self.send_limit_order(
+                vt_orderids: list = self.send_limit_order(
                     strategy,
                     contract,
                     stop_order.direction,
@@ -282,7 +285,7 @@ class CtaEngine(BaseEngine):
                     # Remove from relation map.
                     self.stop_orders.pop(stop_order.stop_orderid)
 
-                    strategy_vt_orderids = self.strategy_orderid_map[strategy.strategy_name]
+                    strategy_vt_orderids: list = self.strategy_orderid_map[strategy.strategy_name]
                     if stop_order.stop_orderid in strategy_vt_orderids:
                         strategy_vt_orderids.remove(stop_order.stop_orderid)
 
@@ -306,12 +309,12 @@ class CtaEngine(BaseEngine):
         type: OrderType,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """
         Send a new order to server.
         """
         # Create request and send order.
-        original_req = OrderRequest(
+        original_req: OrderRequest = OrderRequest(
             symbol=contract.symbol,
             exchange=contract.exchange,
             direction=direction,
@@ -323,13 +326,13 @@ class CtaEngine(BaseEngine):
         )
 
         # Convert with offset converter
-        req_list = self.offset_converter.convert_order_request(original_req, lock, net)
+        req_list: List[OrderRequest] = self.offset_converter.convert_order_request(original_req, lock, net)
 
         # Send Orders
-        vt_orderids = []
+        vt_orderids: list = []
 
         for req in req_list:
-            vt_orderid = self.main_engine.send_order(req, contract.gateway_name)
+            vt_orderid: str = self.main_engine.send_order(req, contract.gateway_name)
 
             # Check if sending order successful
             if not vt_orderid:
@@ -355,7 +358,7 @@ class CtaEngine(BaseEngine):
         volume: float,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """
         Send a limit order to server.
         """
@@ -381,7 +384,7 @@ class CtaEngine(BaseEngine):
         volume: float,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """
         Send a stop order to server.
 
@@ -409,14 +412,14 @@ class CtaEngine(BaseEngine):
         volume: float,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """
         Create a new local stop order.
         """
         self.stop_order_count += 1
-        stop_orderid = f"{STOPORDER_PREFIX}.{self.stop_order_count}"
+        stop_orderid: str = f"{STOPORDER_PREFIX}.{self.stop_order_count}"
 
-        stop_order = StopOrder(
+        stop_order: StopOrder = StopOrder(
             vt_symbol=strategy.vt_symbol,
             direction=direction,
             offset=offset,
@@ -431,7 +434,7 @@ class CtaEngine(BaseEngine):
 
         self.stop_orders[stop_orderid] = stop_order
 
-        vt_orderids = self.strategy_orderid_map[strategy.strategy_name]
+        vt_orderids: list = self.strategy_orderid_map[strategy.strategy_name]
         vt_orderids.add(stop_orderid)
 
         self.call_strategy_func(strategy, strategy.on_stop_order, stop_order)
@@ -439,31 +442,31 @@ class CtaEngine(BaseEngine):
 
         return [stop_orderid]
 
-    def cancel_server_order(self, strategy: CtaTemplate, vt_orderid: str):
+    def cancel_server_order(self, strategy: CtaTemplate, vt_orderid: str) -> None:
         """
         Cancel existing order by vt_orderid.
         """
-        order = self.main_engine.get_order(vt_orderid)
+        order: Optional[OrderData] = self.main_engine.get_order(vt_orderid)
         if not order:
             self.write_log(f"撤单失败，找不到委托{vt_orderid}", strategy)
             return
 
-        req = order.create_cancel_request()
+        req: CancelRequest = order.create_cancel_request()
         self.main_engine.cancel_order(req, order.gateway_name)
 
-    def cancel_local_stop_order(self, strategy: CtaTemplate, stop_orderid: str):
+    def cancel_local_stop_order(self, strategy: CtaTemplate, stop_orderid: str) -> None:
         """
         Cancel a local stop order.
         """
-        stop_order = self.stop_orders.get(stop_orderid, None)
+        stop_order: Optional[StopOrder] = self.stop_orders.get(stop_orderid, None)
         if not stop_order:
             return
-        strategy = self.strategies[stop_order.strategy_name]
+        strategy: CtaTemplate = self.strategies[stop_order.strategy_name]
 
         # Remove from relation map.
         self.stop_orders.pop(stop_orderid)
 
-        vt_orderids = self.strategy_orderid_map[strategy.strategy_name]
+        vt_orderids: list = self.strategy_orderid_map[strategy.strategy_name]
         if stop_orderid in vt_orderids:
             vt_orderids.remove(stop_orderid)
 
@@ -483,17 +486,17 @@ class CtaEngine(BaseEngine):
         stop: bool,
         lock: bool,
         net: bool
-    ):
+    ) -> list:
         """
         """
-        contract = self.main_engine.get_contract(strategy.vt_symbol)
+        contract: Optional[ContractData] = self.main_engine.get_contract(strategy.vt_symbol)
         if not contract:
             self.write_log(f"委托失败，找不到合约：{strategy.vt_symbol}", strategy)
             return ""
 
         # Round order price and volume to nearest incremental value
-        price = round_to(price, contract.pricetick)
-        volume = round_to(volume, contract.min_volume)
+        price: float = round_to(price, contract.pricetick)
+        volume: float = round_to(volume, contract.min_volume)
 
         if stop:
             if contract.stop_supported:
@@ -509,7 +512,7 @@ class CtaEngine(BaseEngine):
                 strategy, contract, direction, offset, price, volume, lock, net
             )
 
-    def cancel_order(self, strategy: CtaTemplate, vt_orderid: str):
+    def cancel_order(self, strategy: CtaTemplate, vt_orderid: str) -> None:
         """
         """
         if vt_orderid.startswith(STOPORDER_PREFIX):
@@ -517,26 +520,26 @@ class CtaEngine(BaseEngine):
         else:
             self.cancel_server_order(strategy, vt_orderid)
 
-    def cancel_all(self, strategy: CtaTemplate):
+    def cancel_all(self, strategy: CtaTemplate) -> None:
         """
         Cancel all active orders of a strategy.
         """
-        vt_orderids = self.strategy_orderid_map[strategy.strategy_name]
+        vt_orderids: list = self.strategy_orderid_map[strategy.strategy_name]
         if not vt_orderids:
             return
 
         for vt_orderid in copy(vt_orderids):
             self.cancel_order(strategy, vt_orderid)
 
-    def get_engine_type(self):
+    def get_engine_type(self) -> EngineType:
         """"""
         return self.engine_type
 
-    def get_pricetick(self, strategy: CtaTemplate):
+    def get_pricetick(self, strategy: CtaTemplate) -> float:
         """
         Return contract pricetick data.
         """
-        contract = self.main_engine.get_contract(strategy.vt_symbol)
+        contract: Optional[ContractData] = self.main_engine.get_contract(strategy.vt_symbol)
 
         if contract:
             return contract.pricetick
@@ -553,31 +556,31 @@ class CtaEngine(BaseEngine):
     ) -> List[BarData]:
         """"""
         symbol, exchange = extract_vt_symbol(vt_symbol)
-        end = datetime.now(LOCAL_TZ)
-        start = end - timedelta(days)
-        bars = []
+        end: datetime = datetime.now(LOCAL_TZ)
+        start: datetime = end - timedelta(days)
+        bars: List[BarData] = []
 
         # Pass gateway and datafeed if use_database set to True
         if not use_database:
             # Query bars from gateway if available
-            contract = self.main_engine.get_contract(vt_symbol)
+            contract: Optional[ContractData] = self.main_engine.get_contract(vt_symbol)
 
             if contract and contract.history_data:
-                req = HistoryRequest(
+                req: HistoryRequest = HistoryRequest(
                     symbol=symbol,
                     exchange=exchange,
                     interval=interval,
                     start=start,
                     end=end
                 )
-                bars = self.main_engine.query_history(req, contract.gateway_name)
+                bars: List[BarData] = self.main_engine.query_history(req, contract.gateway_name)
 
             # Try to query bars from datafeed, if not found, load from database.
             else:
-                bars = self.query_bar_from_datafeed(symbol, exchange, interval, start, end)
+                bars: List[BarData] = self.query_bar_from_datafeed(symbol, exchange, interval, start, end)
 
         if not bars:
-            bars = self.database.load_bar_data(
+            bars: List[BarData] = self.database.load_bar_data(
                 symbol=symbol,
                 exchange=exchange,
                 interval=interval,
@@ -595,10 +598,10 @@ class CtaEngine(BaseEngine):
     ) -> List[TickData]:
         """"""
         symbol, exchange = extract_vt_symbol(vt_symbol)
-        end = datetime.now(LOCAL_TZ)
-        start = end - timedelta(days)
+        end: datetime = datetime.now(LOCAL_TZ)
+        start: datetime = end - timedelta(days)
 
-        ticks = self.database.load_tick_data(
+        ticks: List[TickData] = self.database.load_tick_data(
             symbol=symbol,
             exchange=exchange,
             start=start,
@@ -609,7 +612,7 @@ class CtaEngine(BaseEngine):
 
     def call_strategy_func(
         self, strategy: CtaTemplate, func: Callable, params: Any = None
-    ):
+    ) -> None:
         """
         Call function of a strategy and catch any exception raised.
         """
@@ -622,12 +625,12 @@ class CtaEngine(BaseEngine):
             strategy.trading = False
             strategy.inited = False
 
-            msg = f"触发异常已停止\n{traceback.format_exc()}"
+            msg: str = f"触发异常已停止\n{traceback.format_exc()}"
             self.write_log(msg, strategy)
 
     def add_strategy(
         self, class_name: str, strategy_name: str, vt_symbol: str, setting: dict
-    ):
+    ) -> None:
         """
         Add a new strategy.
         """
@@ -635,7 +638,7 @@ class CtaEngine(BaseEngine):
             self.write_log(f"创建策略失败，存在重名{strategy_name}")
             return
 
-        strategy_class = self.classes.get(class_name, None)
+        strategy_class: Optional[Type[CtaTemplate]] = self.classes.get(class_name, None)
         if not strategy_class:
             self.write_log(f"创建策略失败，找不到策略类{class_name}")
             return
@@ -649,11 +652,11 @@ class CtaEngine(BaseEngine):
             self.write_log("创建策略失败，本地代码的交易所后缀不正确")
             return
 
-        strategy = strategy_class(self, strategy_name, vt_symbol, setting)
+        strategy: CtaTemplate = strategy_class(self, strategy_name, vt_symbol, setting)
         self.strategies[strategy_name] = strategy
 
         # Add vt_symbol to strategy map.
-        strategies = self.symbol_strategy_map[vt_symbol]
+        strategies: list = self.symbol_strategy_map[vt_symbol]
         strategies.append(strategy)
 
         # Update to setting file.
@@ -661,17 +664,17 @@ class CtaEngine(BaseEngine):
 
         self.put_strategy_event(strategy)
 
-    def init_strategy(self, strategy_name: str):
+    def init_strategy(self, strategy_name: str) -> Future:
         """
         Init a strategy.
         """
-        self.init_executor.submit(self._init_strategy, strategy_name)
+        return self.init_executor.submit(self._init_strategy, strategy_name)
 
-    def _init_strategy(self, strategy_name: str):
+    def _init_strategy(self, strategy_name: str) -> None:
         """
         Init strategies in queue.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
 
         if strategy.inited:
             self.write_log(f"{strategy_name}已经完成初始化，禁止重复操作")
@@ -683,7 +686,7 @@ class CtaEngine(BaseEngine):
         self.call_strategy_func(strategy, strategy.on_init)
 
         # Restore strategy data(variables)
-        data = self.strategy_data.get(strategy_name, None)
+        data: Optional[dict] = self.strategy_data.get(strategy_name, None)
         if data:
             for name in strategy.variables:
                 value = data.get(name, None)
@@ -691,9 +694,9 @@ class CtaEngine(BaseEngine):
                     setattr(strategy, name, value)
 
         # Subscribe market data
-        contract = self.main_engine.get_contract(strategy.vt_symbol)
+        contract: Optional[ContractData] = self.main_engine.get_contract(strategy.vt_symbol)
         if contract:
-            req = SubscribeRequest(
+            req: SubscribeRequest = SubscribeRequest(
                 symbol=contract.symbol, exchange=contract.exchange)
             self.main_engine.subscribe(req, contract.gateway_name)
         else:
@@ -704,11 +707,11 @@ class CtaEngine(BaseEngine):
         self.put_strategy_event(strategy)
         self.write_log(f"{strategy_name}初始化完成")
 
-    def start_strategy(self, strategy_name: str):
+    def start_strategy(self, strategy_name: str) -> None:
         """
         Start a strategy.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
         if not strategy.inited:
             self.write_log(f"策略{strategy.strategy_name}启动失败，请先初始化")
             return
@@ -722,11 +725,11 @@ class CtaEngine(BaseEngine):
 
         self.put_strategy_event(strategy)
 
-    def stop_strategy(self, strategy_name: str):
+    def stop_strategy(self, strategy_name: str) -> None:
         """
         Stop a strategy.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
         if not strategy.trading:
             return
 
@@ -745,21 +748,21 @@ class CtaEngine(BaseEngine):
         # Update GUI
         self.put_strategy_event(strategy)
 
-    def edit_strategy(self, strategy_name: str, setting: dict):
+    def edit_strategy(self, strategy_name: str, setting: dict) -> None:
         """
         Edit parameters of a strategy.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
         strategy.update_setting(setting)
 
         self.update_strategy_setting(strategy_name, setting)
         self.put_strategy_event(strategy)
 
-    def remove_strategy(self, strategy_name: str):
+    def remove_strategy(self, strategy_name: str) -> bool:
         """
         Remove a strategy.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
         if strategy.trading:
             self.write_log(f"策略{strategy.strategy_name}移除失败，请先停止")
             return
@@ -768,12 +771,12 @@ class CtaEngine(BaseEngine):
         self.remove_strategy_setting(strategy_name)
 
         # Remove from symbol strategy map
-        strategies = self.symbol_strategy_map[strategy.vt_symbol]
+        strategies: list = self.symbol_strategy_map[strategy.vt_symbol]
         strategies.remove(strategy)
 
         # Remove from active orderid map
         if strategy_name in self.strategy_orderid_map:
-            vt_orderids = self.strategy_orderid_map.pop(strategy_name)
+            vt_orderids: list = self.strategy_orderid_map.pop(strategy_name)
 
             # Remove vt_orderid strategy map
             for vt_orderid in vt_orderids:
@@ -786,33 +789,33 @@ class CtaEngine(BaseEngine):
         self.write_log(f"策略{strategy.strategy_name}移除移除成功")
         return True
 
-    def load_strategy_class(self):
+    def load_strategy_class(self) -> None:
         """
         Load strategy class from source code.
         """
-        path1 = Path(__file__).parent.joinpath("strategies")
+        path1: Path = Path(__file__).parent.joinpath("strategies")
         self.load_strategy_class_from_folder(path1, "vnpy_ctastrategy.strategies")
 
-        path2 = Path.cwd().joinpath("strategies")
+        path2: Path = Path.cwd().joinpath("strategies")
         self.load_strategy_class_from_folder(path2, "strategies")
 
-    def load_strategy_class_from_folder(self, path: Path, module_name: str = ""):
+    def load_strategy_class_from_folder(self, path: Path, module_name: str = "") -> None:
         """
         Load strategy class from certain folder.
         """
         for suffix in ["py", "pyd", "so"]:
             pathname: str = str(path.joinpath(f"*.{suffix}"))
             for filepath in glob(pathname):
-                filename: str = Path(filepath).stem
+                filename = Path(filepath).stem
                 name: str = f"{module_name}.{filename}"
                 self.load_strategy_class_from_module(name)
 
-    def load_strategy_class_from_module(self, module_name: str):
+    def load_strategy_class_from_module(self, module_name: str) -> None:
         """
         Load strategy class from module file.
         """
         try:
-            module = importlib.import_module(module_name)
+            module: ModuleType = importlib.import_module(module_name)
 
             # 重载模块，确保如果策略文件中有任何修改，能够立即生效。
             importlib.reload(module)
@@ -822,70 +825,72 @@ class CtaEngine(BaseEngine):
                 if (isinstance(value, type) and issubclass(value, CtaTemplate) and value is not CtaTemplate):
                     self.classes[value.__name__] = value
         except:  # noqa
-            msg = f"策略文件{module_name}加载失败，触发异常：\n{traceback.format_exc()}"
+            msg: str = f"策略文件{module_name}加载失败，触发异常：\n{traceback.format_exc()}"
             self.write_log(msg)
 
-    def load_strategy_data(self):
+    def load_strategy_data(self) -> None:
         """
         Load strategy data from json file.
         """
         self.strategy_data = load_json(self.data_filename)
 
-    def sync_strategy_data(self, strategy: CtaTemplate):
+    def sync_strategy_data(self, strategy: CtaTemplate) -> None:
         """
         Sync strategy data into json file.
         """
-        data = strategy.get_variables()
+        data: dict = strategy.get_variables()
         data.pop("inited")      # Strategy status (inited, trading) should not be synced.
         data.pop("trading")
 
         self.strategy_data[strategy.strategy_name] = data
         save_json(self.data_filename, self.strategy_data)
 
-    def get_all_strategy_class_names(self):
+    def get_all_strategy_class_names(self) -> list:
         """
         Return names of strategy classes loaded.
         """
         return list(self.classes.keys())
 
-    def get_strategy_class_parameters(self, class_name: str):
+    def get_strategy_class_parameters(self, class_name: str) -> dict:
         """
         Get default parameters of a strategy class.
         """
-        strategy_class = self.classes[class_name]
+        strategy_class: Type[CtaTemplate] = self.classes[class_name]
 
-        parameters = {}
+        parameters: dict = {}
         for name in strategy_class.parameters:
             parameters[name] = getattr(strategy_class, name)
 
         return parameters
 
-    def get_strategy_parameters(self, strategy_name):
+    def get_strategy_parameters(self, strategy_name) -> dict:
         """
         Get parameters of a strategy.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
         return strategy.get_parameters()
 
-    def init_all_strategies(self):
+    def init_all_strategies(self) -> Dict[str, Future]:
         """
         """
+        futures: Dict[str, Future] = {}
         for strategy_name in self.strategies.keys():
-            self.init_strategy(strategy_name)
+            futures[strategy_name] = self.init_strategy(strategy_name)
+        return futures
 
-    def start_all_strategies(self):
+    def start_all_strategies(self) -> None:
         """
         """
         for strategy_name in self.strategies.keys():
             self.start_strategy(strategy_name)
 
-    def stop_all_strategies(self):
+    def stop_all_strategies(self) -> None:
         """
         """
         for strategy_name in self.strategies.keys():
             self.stop_strategy(strategy_name)
 
-    def load_strategy_setting(self):
+    def load_strategy_setting(self) -> None:
         """
         Load setting file.
         """
@@ -899,11 +904,11 @@ class CtaEngine(BaseEngine):
                 strategy_config["setting"]
             )
 
-    def update_strategy_setting(self, strategy_name: str, setting: dict):
+    def update_strategy_setting(self, strategy_name: str, setting: dict) -> None:
         """
         Update setting file.
         """
-        strategy = self.strategies[strategy_name]
+        strategy: CtaTemplate = self.strategies[strategy_name]
 
         self.strategy_setting[strategy_name] = {
             "class_name": strategy.__class__.__name__,
@@ -912,7 +917,7 @@ class CtaEngine(BaseEngine):
         }
         save_json(self.setting_filename, self.strategy_setting)
 
-    def remove_strategy_setting(self, strategy_name: str):
+    def remove_strategy_setting(self, strategy_name: str) -> None:
         """
         Update setting file.
         """
@@ -922,39 +927,39 @@ class CtaEngine(BaseEngine):
         self.strategy_setting.pop(strategy_name)
         save_json(self.setting_filename, self.strategy_setting)
 
-    def put_stop_order_event(self, stop_order: StopOrder):
+    def put_stop_order_event(self, stop_order: StopOrder) -> None:
         """
         Put an event to update stop order status.
         """
-        event = Event(EVENT_CTA_STOPORDER, stop_order)
+        event: Event = Event(EVENT_CTA_STOPORDER, stop_order)
         self.event_engine.put(event)
 
-    def put_strategy_event(self, strategy: CtaTemplate):
+    def put_strategy_event(self, strategy: CtaTemplate) -> None:
         """
         Put an event to update strategy status.
         """
-        data = strategy.get_data()
-        event = Event(EVENT_CTA_STRATEGY, data)
+        data: dict = strategy.get_data()
+        event: Event = Event(EVENT_CTA_STRATEGY, data)
         self.event_engine.put(event)
 
-    def write_log(self, msg: str, strategy: CtaTemplate = None):
+    def write_log(self, msg: str, strategy: CtaTemplate = None) -> None:
         """
         Create cta engine log event.
         """
         if strategy:
-            msg = f"[{strategy.strategy_name}]  {msg}"
+            msg: str = f"[{strategy.strategy_name}]  {msg}"
 
-        log = LogData(msg=msg, gateway_name=APP_NAME)
-        event = Event(type=EVENT_CTA_LOG, data=log)
+        log: LogData = LogData(msg=msg, gateway_name=APP_NAME)
+        event: Event = Event(type=EVENT_CTA_LOG, data=log)
         self.event_engine.put(event)
 
-    def send_email(self, msg: str, strategy: CtaTemplate = None):
+    def send_email(self, msg: str, strategy: CtaTemplate = None) -> None:
         """
         Send email to default receiver.
         """
         if strategy:
-            subject = f"{strategy.strategy_name}"
+            subject: str = f"{strategy.strategy_name}"
         else:
-            subject = "CTA策略引擎"
+            subject: str = "CTA策略引擎"
 
         self.main_engine.send_email(subject, msg)
