@@ -1,7 +1,6 @@
-""""""
 from abc import ABC
 from copy import copy
-from typing import Any, Callable
+from typing import Any, Callable, List
 
 from vnpy.trader.constant import Interval, Direction, Offset
 from vnpy.trader.object import BarData, TickData, OrderData, TradeData
@@ -13,9 +12,9 @@ from .base import StopOrder, EngineType
 class CtaTemplate(ABC):
     """"""
 
-    author = ""
-    parameters = []
-    variables = []
+    author: str = ""
+    parameters: list = []
+    variables: list = []
 
     def __init__(
         self,
@@ -23,15 +22,15 @@ class CtaTemplate(ABC):
         strategy_name: str,
         vt_symbol: str,
         setting: dict,
-    ):
+    ) -> None:
         """"""
-        self.cta_engine = cta_engine
-        self.strategy_name = strategy_name
-        self.vt_symbol = vt_symbol
+        self.cta_engine: Any = cta_engine
+        self.strategy_name: str = strategy_name
+        self.vt_symbol: str = vt_symbol
 
-        self.inited = False
-        self.trading = False
-        self.pos = 0
+        self.inited: bool = False
+        self.trading: bool = False
+        self.pos: int = 0
 
         # Copy a new variables list here to avoid duplicate insert when multiple
         # strategy instances are created with the same strategy class.
@@ -42,7 +41,7 @@ class CtaTemplate(ABC):
 
         self.update_setting(setting)
 
-    def update_setting(self, setting: dict):
+    def update_setting(self, setting: dict) -> None:
         """
         Update strategy parameter wtih value in setting dict.
         """
@@ -51,38 +50,38 @@ class CtaTemplate(ABC):
                 setattr(self, name, setting[name])
 
     @classmethod
-    def get_class_parameters(cls):
+    def get_class_parameters(cls) -> dict:
         """
         Get default parameters dict of strategy class.
         """
-        class_parameters = {}
+        class_parameters: dict = {}
         for name in cls.parameters:
             class_parameters[name] = getattr(cls, name)
         return class_parameters
 
-    def get_parameters(self):
+    def get_parameters(self) -> dict:
         """
         Get strategy parameters dict.
         """
-        strategy_parameters = {}
+        strategy_parameters: dict = {}
         for name in self.parameters:
             strategy_parameters[name] = getattr(self, name)
         return strategy_parameters
 
-    def get_variables(self):
+    def get_variables(self) -> dict:
         """
         Get strategy variables dict.
         """
-        strategy_variables = {}
+        strategy_variables: dict = {}
         for name in self.variables:
             strategy_variables[name] = getattr(self, name)
         return strategy_variables
 
-    def get_data(self):
+    def get_data(self) -> dict:
         """
         Get strategy data.
         """
-        strategy_data = {
+        strategy_data: dict = {
             "strategy_name": self.strategy_name,
             "vt_symbol": self.vt_symbol,
             "class_name": self.__class__.__name__,
@@ -93,56 +92,56 @@ class CtaTemplate(ABC):
         return strategy_data
 
     @virtual
-    def on_init(self):
+    def on_init(self) -> None:
         """
         Callback when strategy is inited.
         """
         pass
 
     @virtual
-    def on_start(self):
+    def on_start(self) -> None:
         """
         Callback when strategy is started.
         """
         pass
 
     @virtual
-    def on_stop(self):
+    def on_stop(self) -> None:
         """
         Callback when strategy is stopped.
         """
         pass
 
     @virtual
-    def on_tick(self, tick: TickData):
+    def on_tick(self, tick: TickData) -> None:
         """
         Callback of new tick data update.
         """
         pass
 
     @virtual
-    def on_bar(self, bar: BarData):
+    def on_bar(self, bar: BarData) -> None:
         """
         Callback of new bar data update.
         """
         pass
 
     @virtual
-    def on_trade(self, trade: TradeData):
+    def on_trade(self, trade: TradeData) -> None:
         """
         Callback of new trade data update.
         """
         pass
 
     @virtual
-    def on_order(self, order: OrderData):
+    def on_order(self, order: OrderData) -> None:
         """
         Callback of new order data update.
         """
         pass
 
     @virtual
-    def on_stop_order(self, stop_order: StopOrder):
+    def on_stop_order(self, stop_order: StopOrder) -> None:
         """
         Callback of stop order update.
         """
@@ -155,7 +154,7 @@ class CtaTemplate(ABC):
         stop: bool = False,
         lock: bool = False,
         net: bool = False
-    ):
+    ) -> list:
         """
         Send buy order to open a long position.
         """
@@ -176,7 +175,7 @@ class CtaTemplate(ABC):
         stop: bool = False,
         lock: bool = False,
         net: bool = False
-    ):
+    ) -> list:
         """
         Send sell order to close a long position.
         """
@@ -197,7 +196,7 @@ class CtaTemplate(ABC):
         stop: bool = False,
         lock: bool = False,
         net: bool = False
-    ):
+    ) -> list:
         """
         Send short order to open as short position.
         """
@@ -218,7 +217,7 @@ class CtaTemplate(ABC):
         stop: bool = False,
         lock: bool = False,
         net: bool = False
-    ):
+    ) -> list:
         """
         Send cover order to close a short position.
         """
@@ -241,45 +240,45 @@ class CtaTemplate(ABC):
         stop: bool = False,
         lock: bool = False,
         net: bool = False
-    ):
+    ) -> list:
         """
         Send a new order.
         """
         if self.trading:
-            vt_orderids = self.cta_engine.send_order(
+            vt_orderids: list = self.cta_engine.send_order(
                 self, direction, offset, price, volume, stop, lock, net
             )
             return vt_orderids
         else:
             return []
 
-    def cancel_order(self, vt_orderid: str):
+    def cancel_order(self, vt_orderid: str) -> None:
         """
         Cancel an existing order.
         """
         if self.trading:
             self.cta_engine.cancel_order(self, vt_orderid)
 
-    def cancel_all(self):
+    def cancel_all(self) -> None:
         """
         Cancel all orders sent by strategy.
         """
         if self.trading:
             self.cta_engine.cancel_all(self)
 
-    def write_log(self, msg: str):
+    def write_log(self, msg: str) -> None:
         """
         Write a log message.
         """
         self.cta_engine.write_log(msg, self)
 
-    def get_engine_type(self):
+    def get_engine_type(self) -> EngineType:
         """
         Return whether the cta_engine is backtesting or live trading.
         """
         return self.cta_engine.get_engine_type()
 
-    def get_pricetick(self):
+    def get_pricetick(self) -> float:
         """
         Return pricetick data of trading contract.
         """
@@ -291,14 +290,14 @@ class CtaTemplate(ABC):
         interval: Interval = Interval.MINUTE,
         callback: Callable = None,
         use_database: bool = False
-    ):
+    ) -> None:
         """
         Load historical bar data for initializing strategy.
         """
         if not callback:
-            callback = self.on_bar
+            callback: Callable = self.on_bar
 
-        bars = self.cta_engine.load_bar(
+        bars: List[BarData] = self.cta_engine.load_bar(
             self.vt_symbol,
             days,
             interval,
@@ -309,30 +308,30 @@ class CtaTemplate(ABC):
         for bar in bars:
             callback(bar)
 
-    def load_tick(self, days: int):
+    def load_tick(self, days: int) -> None:
         """
         Load historical tick data for initializing strategy.
         """
-        ticks = self.cta_engine.load_tick(self.vt_symbol, days, self.on_tick)
+        ticks: List[TickData] = self.cta_engine.load_tick(self.vt_symbol, days, self.on_tick)
 
         for tick in ticks:
             self.on_tick(tick)
 
-    def put_event(self):
+    def put_event(self) -> None:
         """
         Put an strategy data event for ui update.
         """
         if self.inited:
             self.cta_engine.put_strategy_event(self)
 
-    def send_email(self, msg):
+    def send_email(self, msg) -> None:
         """
         Send email to default receiver.
         """
         if self.inited:
             self.cta_engine.send_email(msg, self)
 
-    def sync_data(self):
+    def sync_data(self) -> None:
         """
         Sync strategy variables value into disk storage.
         """
@@ -343,29 +342,29 @@ class CtaTemplate(ABC):
 class CtaSignal(ABC):
     """"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """"""
         self.signal_pos = 0
 
     @virtual
-    def on_tick(self, tick: TickData):
+    def on_tick(self, tick: TickData) -> None:
         """
         Callback of new tick data update.
         """
         pass
 
     @virtual
-    def on_bar(self, bar: BarData):
+    def on_bar(self, bar: BarData) -> None:
         """
         Callback of new bar data update.
         """
         pass
 
-    def set_signal_pos(self, pos):
+    def set_signal_pos(self, pos) -> None:
         """"""
         self.signal_pos = pos
 
-    def get_signal_pos(self):
+    def get_signal_pos(self) -> Any:
         """"""
         return self.signal_pos
 
@@ -374,21 +373,21 @@ class TargetPosTemplate(CtaTemplate):
     """"""
     tick_add = 1
 
-    last_tick = None
-    last_bar = None
+    last_tick: TickData = None
+    last_bar: BarData = None
     target_pos = 0
 
-    def __init__(self, cta_engine, strategy_name, vt_symbol, setting):
+    def __init__(self, cta_engine, strategy_name, vt_symbol, setting) -> None:
         """"""
         super().__init__(cta_engine, strategy_name, vt_symbol, setting)
 
-        self.active_orderids = []
-        self.cancel_orderids = []
+        self.active_orderids: list = []
+        self.cancel_orderids: list = []
 
         self.variables.append("target_pos")
 
     @virtual
-    def on_tick(self, tick: TickData):
+    def on_tick(self, tick: TickData) -> None:
         """
         Callback of new tick data update.
         """
@@ -398,18 +397,18 @@ class TargetPosTemplate(CtaTemplate):
             self.trade()
 
     @virtual
-    def on_bar(self, bar: BarData):
+    def on_bar(self, bar: BarData) -> None:
         """
         Callback of new bar data update.
         """
         self.last_bar = bar
 
     @virtual
-    def on_order(self, order: OrderData):
+    def on_order(self, order: OrderData) -> None:
         """
         Callback of new order data update.
         """
-        vt_orderid = order.vt_orderid
+        vt_orderid: str = order.vt_orderid
 
         if not order.is_active():
             if vt_orderid in self.active_orderids:
@@ -418,33 +417,33 @@ class TargetPosTemplate(CtaTemplate):
             if vt_orderid in self.cancel_orderids:
                 self.cancel_orderids.remove(vt_orderid)
 
-    def check_order_finished(self):
+    def check_order_finished(self) -> bool:
         """"""
         if self.active_orderids:
             return False
         else:
             return True
 
-    def set_target_pos(self, target_pos):
+    def set_target_pos(self, target_pos) -> None:
         """"""
         self.target_pos = target_pos
         self.trade()
 
-    def trade(self):
+    def trade(self) -> None:
         """"""
         if not self.check_order_finished():
             self.cancel_old_order()
         else:
             self.send_new_order()
 
-    def cancel_old_order(self):
+    def cancel_old_order(self) -> None:
         """"""
         for vt_orderid in self.active_orderids:
             if vt_orderid not in self.cancel_orderids:
                 self.cancel_order(vt_orderid)
                 self.cancel_orderids.append(vt_orderid)
 
-    def send_new_order(self):
+    def send_new_order(self) -> None:
         """"""
         pos_change = self.target_pos - self.pos
         if not pos_change:
@@ -471,9 +470,9 @@ class TargetPosTemplate(CtaTemplate):
 
         if self.get_engine_type() == EngineType.BACKTESTING:
             if pos_change > 0:
-                vt_orderids = self.buy(long_price, abs(pos_change))
+                vt_orderids: list = self.buy(long_price, abs(pos_change))
             else:
-                vt_orderids = self.short(short_price, abs(pos_change))
+                vt_orderids: list = self.short(short_price, abs(pos_change))
             self.active_orderids.extend(vt_orderids)
 
         else:
@@ -483,17 +482,17 @@ class TargetPosTemplate(CtaTemplate):
             if pos_change > 0:
                 if self.pos < 0:
                     if pos_change < abs(self.pos):
-                        vt_orderids = self.cover(long_price, pos_change)
+                        vt_orderids: list = self.cover(long_price, pos_change)
                     else:
-                        vt_orderids = self.cover(long_price, abs(self.pos))
+                        vt_orderids: list = self.cover(long_price, abs(self.pos))
                 else:
-                    vt_orderids = self.buy(long_price, abs(pos_change))
+                    vt_orderids: list = self.buy(long_price, abs(pos_change))
             else:
                 if self.pos > 0:
                     if abs(pos_change) < self.pos:
-                        vt_orderids = self.sell(short_price, abs(pos_change))
+                        vt_orderids: list = self.sell(short_price, abs(pos_change))
                     else:
-                        vt_orderids = self.sell(short_price, abs(self.pos))
+                        vt_orderids: list = self.sell(short_price, abs(self.pos))
                 else:
-                    vt_orderids = self.short(short_price, abs(pos_change))
+                    vt_orderids: list = self.short(short_price, abs(pos_change))
             self.active_orderids.extend(vt_orderids)
